package cn.org.bjca.trust.pushdemo.message;

import cn.org.bjca.trust.pushdemo.json.GsonImplHelp;
import org.apache.tomcat.util.codec.binary.Base64;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.http.HttpHeaders;
import org.springframework.util.MultiValueMap;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.Objects;

public class ImClient implements ImClientInterface {

    private boolean isConnecting = false;

    private MqttClient client;
    private MqttConnectOptions options;

    @Override
    public void connect() {
        init();
        mConnect();
    }

    @Override
    public void reConnect() {
        init();
        mConnect();
    }

    @Override
    public void disConnect() {

    }

    @Override
    public boolean isConnect() {
        return client.isConnected();
    }

    @Override
    public boolean isConnecting() {
        return isConnecting;
    }

    @Override
    public void sendMessage(SendMessage msg) {

    }

    private void init() {
        if (null == this.client) {
            try {
                this.client = new MqttClient("tcp://114.115.203.60:18883", "server" + System.currentTimeMillis());
            } catch (MqttException e) {
                throw new RuntimeException(e);
            }
            options = new MqttConnectOptions();
            options.setCleanSession(true);
            options.setUserName("server");
            options.setPassword("server".toCharArray());
            options.setAutomaticReconnect(true);
            options.setKeepAliveInterval(20);
            options.setConnectionTimeout(30);
            client.setCallback(new MqttCallbackExtended() {
                @Override
                public void connectComplete(boolean b, String s) {
                    System.out.println("----------------------连接成功--------------------------" + b);
                    isConnecting = false;
                    MqttClient client1 = client;
                    try {
                        client1.subscribe("server", 2);
                    } catch (MqttException e) {
                        throw new RuntimeException(e);
                    }
                }

                @Override
                public void connectionLost(Throwable throwable) {
                    System.out.println("----------------------掉线--------------------------");
                }

                final String enc = new String(Base64.encodeBase64("admin:admin".getBytes(StandardCharsets.ISO_8859_1), false));
                HttpHeaders headers = null;

                private MultiValueMap<String, String> getHeaders() {

                    if (headers == null) {
                        headers = new HttpHeaders();
                        headers.add(HttpHeaders.AUTHORIZATION, "Basic " + enc);
                        headers.add(HttpHeaders.CONTENT_TYPE, "application/json");
                    }
                    return headers;
                }

                @Override
                public void messageArrived(String s, MqttMessage mqttMessage) {
//                    System.out.println(new String(mqttMessage.getPayload()));
                    String msg = new String(mqttMessage.getPayload());

                    BaseMessage message = GsonImplHelp.get().toObject(msg, PingMessage.class);

                    switch (message.getPacketType()) {
                        case SEND:
                            SendMessage sendMessage = GsonImplHelp.get().toObject(msg, SendMessage.class);
                            try {
                                System.out.println("----------------------消息到达--------------------------" + msg);
                                client.publish(sendMessage.getFromClientId(), GsonImplHelp.get().toJson(new AckSendMessage(sendMessage.getMessageId())).getBytes(StandardCharsets.UTF_8), 2, false);
                            } catch (MqttException e) {
                                System.out.printf("---------------" + e.getMessage());
                            }
                            try {
                                if (sendMessage.getToClientId().startsWith("G_")) {


                                    WebClient webClient = WebClient.builder().baseUrl("http://172.17.0.3:15672").defaultHeaders(httpHeaders -> httpHeaders.addAll(getHeaders())).build();

                                    Mono<ImUser[]> m = webClient.get().uri("/api/users/")  // 请求路径
                                            .retrieve().bodyToMono(ImUser[].class);

                                    for (ImUser imUser : Objects.requireNonNull(m.block())) {
                                        SendMessage message1 = new SendMessage();
                                        message1.setMessageId(sendMessage.getMessageId());
                                        message1.setMessage(sendMessage.getMessage());
                                        message1.setMessageType(sendMessage.getMessageType());
                                        message1.setDescribe(sendMessage.getDescribe());
                                        message1.setFromClientId(sendMessage.getToClientId());
                                        message1.setTarget(sendMessage.getFromClientId());
                                        message1.setToClientId(imUser.getName());
                                        message1.setTimestamp(sendMessage.getTimestamp());
                                        message1.setTenantNo(sendMessage.getTenantNo());
                                        client.publish(message1.getToClientId(), GsonImplHelp.get().toJson(message1).getBytes(StandardCharsets.UTF_8), 2, false);

                                    }

                                } else {
                                    client.publish(sendMessage.getToClientId(), GsonImplHelp.get().toJson(sendMessage).getBytes(StandardCharsets.UTF_8), 2, false);
                                }
                            } catch (MqttException e) {
                                System.out.printf("---------------" + e.getMessage());
                            }
                            System.out.println("----------------------转发完成--------------------------");
                            break;
                        case SENDACK:
                            System.out.println("-------------消息送达---------------------");
                            break;
                        case PING:
                            try {
                                PingMessage pingMessage = GsonImplHelp.get().toObject(msg, PingMessage.class);
                                client.publish(pingMessage.getFromClientId(), GsonImplHelp.get().toJson(new PangMessage()).getBytes(StandardCharsets.UTF_8), 2, false);
                            } catch (MqttException e) {
                            }
                            break;
                        case DISCONNECT:
                            System.out.println("-------------用户注销---------------------");
                            break;
                        case CONNECT:
                            System.out.println("-------------用户连接请求---------------------");
                            ConnectMessage connectMessage = GsonImplHelp.get().toObject(msg, ConnectMessage.class);
                            try {
                                client.publish(connectMessage.getClientId(), GsonImplHelp.get().toJson(new ConnAckMessage()).getBytes(StandardCharsets.UTF_8), 2, false);
                            } catch (MqttException e) {
                            }
                            break;
                    }

                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {

                }
            });
        }
    }

    private void mConnect() {
        if (isConnect() || isConnecting) return;
        isConnecting = true;
        if (!client.isConnected()) {
            System.out.println("---------------------开始连接2----------------------");
            try {
                client.connect(options);
            } catch (MqttException e) {
                System.out.println("---------------------连接失败----------------------"+e.getMessage());
            }
            this.isConnecting = false;
        }

    }
}
